#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2015, 2016 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

UNINSTALL_CMD="/bin/rpm -evh"

# write current status of upgrade to status file
# to be used in upgrade recovery
# arg 1 = status identifier
set_upgrade_status()
{
   /bin/sed -i '/status/d' ${BACKUP_DIR}/status.txt
   /bin/echo "status=$1" >> ${BACKUP_DIR}/status.txt
   /bin/sync
}

upgrade_settings()
{
   INSTALLED_VERSION=$(get_installed_version)
   # set HOST_NAME to the management_ip value
   # from previous version
   HOST_NAME=$(get_config_value ${BACKUP_DIR}/etc/nova/nova.conf management_ip)
   # in 1.2.3 the management_ip didn't exist, so use a different property there
   if [ "$HOST_NAME" == "" ]; then
      HOST_NAME=$(get_config_value ${BACKUP_DIR}/etc/nova/nova.conf osapi_compute_link_prefix)
      HOST_NAME=${HOST_NAME:8:${#HOST_NAME}-13}
   fi
   msg_log "Management_ip: $HOST_NAME"

   upgrade_set_offering

   # status file for upgrade recovery
   if [ -z "$INSTALLED_VERSION" ] || [ "${INSTALLED_VERSION}xxx" != "xxx" ]; then
      for blockedversion in ${NOTSUPPORTED_VERSION}; do
         if( is_pvc_blocked $blockedversion ); then
            eval_gettext "MSG310 \$INSTALLED_VERSION"; /bin/echo
            exit ${RC_VERSION_NOTSUPP}
         fi
      done
      if [ ! -f ${BACKUP_DIR}/status.txt ]; then
         /bin/mkdir -p ${BACKUP_DIR}
         /bin/echo "version=${INSTALLED_VERSION}" > ${BACKUP_DIR}/status.txt
         set_upgrade_status backup_started
      fi
   fi
}

upgrade_set_offering()
{
   if (is_valid_version_file); then
      OFFERING_PREV=$(get_config_value ${VERSION_FILE} offering)
      msg_log $(eval_gettext "MSG262 \$OFFERING_PREV"; /bin/echo)
      if [ "$OFFERING_PREV" == "express" ]; then
         eval_gettext "MSG308 \$OFFERING_PREV"; /bin/echo
         exit ${RC_UPGRADE_NO_OFFERING}
      fi
      if [ ${SILENT} -eq 1 ]; then
         if [ "$OFFERING" == "cloud_powervm" ]; then
             OFFERING_PREV=$OFFERING
         fi
         if [ "$OFFERING_PREV" != "$OFFERING" ]; then
            eval_gettext "MSG272 \$OFFERING \$OFFERING_PREV"; /bin/echo
            exit ${RC_BAD_USAGE}
         fi
      fi
      OFFERING=$OFFERING_PREV     
   else
      die ${RC_UPGRADE_NO_OFFERING}
   fi
}

upgrade_set_props()
{
   INSTALL_TYPE=$UPGRADE
   msg_log "Setting upgrade properties."
   # backup the nova conf file for use in bad path recovery
   if [[ ! -e ${BACKUP_DIR}/etc/nova/nova.conf ]]; then
      /usr/bin/mkdir -p ${BACKUP_DIR}/etc/nova/
      /usr/bin/cp -pf /etc/nova/nova.conf ${BACKUP_DIR}/etc/nova/
   fi
   # set the properties from the previous version that need to be
   # persisted during this upgrade install
   # install ip option
   # install interface
   IPTYPE=$(get_config_value ${BACKUP_DIR}/etc/nova/nova.conf install_ip_option)
   msg_log "Install type: $IPTYPE"
   HOST_INTERFACE_NOVACONF=$(get_config_value ${BACKUP_DIR}/etc/nova/nova.conf install_ip_interface)
   if [ ! -z $HOST_INTERFACE_NOVACONF ]; then
       HOST_INTERFACE=$HOST_INTERFACE_NOVACONF
   fi
   msg_log "Network interface: $HOST_INTERFACE"
}

run_powervc_upgrade()
{
   PVC_BACKUP_DIR="/var/opt/ibm/powervc/backups"
   export PVC_BACKUP_RESTORE=1

   status=$(get_config_value ${BACKUP_DIR}/status.txt status)    

   #We would run backup only if:
   #a)backup is not already completed (backup_complete)
   #b)Uninstall has been started      (uninstall_started)
   #c)Uninstall has been completed    (uninstall_complete)
   if [ "$status" == "backup_started" ]; then
      # Before we do the export we need to upgrade sqlachemy to get the constraints
      local ARCH=$(/bin/uname -m)
      /bin/rpm -U --nodeps ${BASE_DIR}/packages/osee/$ARCH/python-sqlalchemy-*.rpm >> ${LOGFILE}
      # Now we can start the export/backup process
      /usr/bin/echo
      /bin/gettext -s "Performing product backup."
      eval_gettext "The product backup archive is located \${PVC_BACKUP_DIR}";/bin/echo
      # Before we run the backup, extract the data from the DB2 databases
      ${BASE_DIR}/lib/dbexport.sh --logfile $LOGFILE
      # Now we can finally run the backup of the data
      run_powervc_backup
      # Cleanup the temporary DB export file after the backup is done
      /bin/rm -f /etc/nova/exported_db2_data.dat
   fi

   # always need run the uninstall whether regular upgrade
   # or in bad path recovery
   # just need to use the correct uninstall script for regular or
   # in recovery
   # 
   # if bad path and uninstall completed, will need to use latest uninstall script
   local recovery_status=$(get_config_value ${BACKUP_DIR}/status.txt status)

   set_upgrade_status uninstall_started
   /bin/gettext -s "MSG038"
   /bin/echo " "
   /bin/gettext -s "IBM PowerVC uninstallation"
   /bin/echo " "
   msg_log "Upgrade uninstall started."
   export PVC_UPGRADE_UNINSTALL=1
   if [ "$recovery_status" == "uninstall_complete" ]; then
      msg_log "Using latest uninstall script."
      /bin/bash ${BASE_DIR}/lib/uninstall.sh -fy >/dev/null 2>&1
      uninstall_rc=$?
   elif [[ $INSTALLED_VERSION == 1.2.3.* ]]; then
      msg_log "Using 1.2.3.x uninstall script."
      /bin/bash ${BASE_DIR}/lib/uninstall_123x.sh -fy >/dev/null 2>&1
      uninstall_rc=$?
      ${UNINSTALL_CMD} --nodeps python-urllib3 python-requests >/dev/null 2>&1
   else
      msg_log "Using 1.3.0.x uninstall script."
      /bin/bash ${BASE_DIR}/lib/uninstall_130x.sh -fy >/dev/null 2>&1
      uninstall_rc=$?
   fi
   # removed ibm-db rpms in 1.3.1 so remove them now
   ${UNINSTALL_CMD} --nodeps python-ibm-db-alembic >/dev/null 2>&1

   # in 1310, db2 is removed from product so cleanup up ceilodb2
   /usr/sbin/userdel -fr ceilodb2 2>/dev/null

   if [ $uninstall_rc -ne 0 ]; then
      # uninstall failure
      msg_log "uninstall rc = $uninstall_rc"
      die ${RC_UPGRADE_UNINSTALL}
   fi

   #check if the file /opt/ibm/powervc/data/restore-recovery.dat exists and remove the same
   if [ -e /opt/ibm/powervc/data/restore-recovery.dat ]; then
      /bin/rm -rf /opt/ibm/powervc/data/restore-recovery.dat
   fi

   #check if the file /opt/ibm/powervc/data/known_hosts exists and remove the same
   if [ -e /opt/ibm/powervc/data/known_hosts ]; then
      /bin/rm -rf /opt/ibm/powervc/data/known_hosts
   fi

   set_upgrade_status uninstall_complete
   msg_log "Upgrade uninstall complete."
   msg_log ""

   # Rsyslog is no longer used starting with PowerVC 1.3
   /bin/rm -rf /etc/pki/rsyslog

   # create a lockfile before we begin upgrade process
   /bin/echo "$$" > "${UPGRADE_LOCK_FILE}"    
}

run_powervc_backup()
{
   msg_log "Running powervc-backup."

   # Need to modify one of the python files used as part of the backup because in
   # 1.2.3 we excluded things such as prs_compute_node_status.json that were needed
   /bin/sed -i -e "s/'.json', '.xml'/'policy.json'/" /usr/lib/python2.7/site-packages/powervc_oslo/config/data_mgr.py

   # Backup powerVC		
   /opt/ibm/powervc/bin/powervc-backup --noPrompt
   if [ $? -ne 0 ]; then
      die_lock_log ${RC_PVC_BACKUP_FAILED}
   fi   

   # Save off pvc groups
   save_pvc_groups

   set_upgrade_status backup_complete
   msg_log "Completed powervc-backup."
}

run_powervc_restore()
{
   if [ ! -f ${BACKUP_DIR}/status.txt ]; then
      /bin/echo "status=uninstall_started" > ${BACKUP_DIR}/status.txt
   fi
   
   msg_log "Running powervc-restore."
   # Restore pvc groups
   restore_pvc_groups

   # Restore powerVC
   local restore_options="--noPrompt"
   # if cloud is being added then
   # stop services except noncore
   # do not start the services as part of restore operation
   if [ "${OFFERING}" == "cloud_powervm" ]; then
      restore_options="${restore_options} --upgrading"
      /opt/ibm/powervc/bin/powervc-services noncore stop >> ${LOGFILE}
   fi
   /opt/ibm/powervc/bin/powervc-restore ${restore_options}
   if [ $? -ne 0 ]; then
      die_lock_log ${RC_PVC_RESTORE_FAILED}
   fi   
   /bin/rm -f ${BACKUP_DIR}/status.txt
   msg_log "Completed powervc-restore."
}


run_yum_package_cleanup()
{
   #in bad path - attempt to complete any failed yum transactions
   /usr/bin/package-cleanup -y --noplugins --cleandupes --noscripts >/dev/null 2>&1 && return 0
   return 1
}

xfer_users()
{
   # $1 = group to get users from
   # $2 = group to add users to
   
   # get list of users in group
   user_list=$(/usr/bin/getent group $1 | /usr/bin/awk -F':' '{print $4}')
   msg_log "group $1 user list: $user_list"
   
   # create temp group
   /usr/sbin/groupadd $2 > /dev/null 2>&1
   
   # add the list of users to temp group
   local old_ifs="$IFS"
   IFS=","
   for u in $user_list
   do
      /usr/sbin/usermod -a -G $2 $u
   done
   IFS="$old_ifs"

   user_list=$(/usr/bin/getent group $2 | /usr/bin/awk -F':' '{print $4}')
   msg_log "group $2 user list: $user_list"
}

save_pvc_groups()
{
   # save off the admin, deployer, viewer groups
   # on upgrade to keep around for restore operation
   # the ibmpowervc-identity rpm removes the groups when
   # it is uninstalled as part of the upgrade
   xfer_users admin pvcadmintmp
   xfer_users deployer pvcdeployertmp
   xfer_users viewer pvcviewertmp
}

restore_pvc_groups()
{
   # restore the admin, deployer, viewer groups
   # from the temp groups on upgrade
   /usr/sbin/groupadd admin > /dev/null 2>&1
   /usr/sbin/groupadd deployer > /dev/null 2>&1
   /usr/sbin/groupadd viewer > /dev/null 2>&1

   xfer_users pvcadmintmp admin
   xfer_users pvcdeployertmp deployer
   xfer_users pvcviewertmp viewer

   # remove the temp groups
   local grp_list="pvcadmintmp pvcdeployertmp pvcviewertmp"
   for grp in $grp_list
   do
      /usr/bin/getent group $grp > /dev/null 2>&1 && /usr/sbin/groupdel $grp
   done
}

